# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tensorboard/uploader/proto/export_service.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tensorboard.compat.proto import summary_pb2 as tensorboard_dot_compat_dot_proto_dot_summary__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tensorboard/uploader/proto/export_service.proto',
  package='tensorboard.service',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n/tensorboard/uploader/proto/export_service.proto\x12\x13tensorboard.service\x1a\x1fgoogle/protobuf/timestamp.proto\x1a&tensorboard/compat/proto/summary.proto\"\xad\x01\n\x18StreamExperimentsRequest\x12\x32\n\x0eread_timestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0f\n\x07user_id\x18\x02 \x01(\t\x12\r\n\x05limit\x18\x03 \x01(\x03\x12=\n\x10\x65xperiments_mask\x18\x04 \x01(\x0b\x32#.tensorboard.service.ExperimentMask\"i\n\x19StreamExperimentsResponse\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x12\x34\n\x0b\x65xperiments\x18\x02 \x03(\x0b\x32\x1f.tensorboard.service.Experiment\"\xbe\x01\n\nExperiment\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12/\n\x0b\x63reate_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12/\n\x0bupdate_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x13\n\x0bnum_scalars\x18\x04 \x01(\x03\x12\x10\n\x08num_runs\x18\x05 \x01(\x03\x12\x10\n\x08num_tags\x18\x06 \x01(\x03\"\x88\x01\n\x0e\x45xperimentMask\x12\x13\n\x0b\x63reate_time\x18\x02 \x01(\x08\x12\x13\n\x0bupdate_time\x18\x03 \x01(\x08\x12\x13\n\x0bnum_scalars\x18\x04 \x01(\x08\x12\x10\n\x08num_runs\x18\x05 \x01(\x08\x12\x10\n\x08num_tags\x18\x06 \x01(\x08J\x04\x08\x01\x10\x02R\rexperiment_id\"h\n\x1bStreamExperimentDataRequest\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x32\n\x0eread_timestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\xa5\x02\n\x1cStreamExperimentDataResponse\x12\x10\n\x08tag_name\x18\x01 \x01(\t\x12\x10\n\x08run_name\x18\x02 \x01(\t\x12\x32\n\x0ctag_metadata\x18\x03 \x01(\x0b\x32\x1c.tensorboard.SummaryMetadata\x12N\n\x06points\x18\x04 \x01(\x0b\x32>.tensorboard.service.StreamExperimentDataResponse.ScalarPoints\x1a]\n\x0cScalarPoints\x12\r\n\x05steps\x18\x01 \x03(\x03\x12.\n\nwall_times\x18\x02 \x03(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x0e\n\x06values\x18\x03 \x03(\x01\x32\x95\x02\n\x1aTensorBoardExporterService\x12v\n\x11StreamExperiments\x12-.tensorboard.service.StreamExperimentsRequest\x1a..tensorboard.service.StreamExperimentsResponse\"\x00\x30\x01\x12\x7f\n\x14StreamExperimentData\x12\x30.tensorboard.service.StreamExperimentDataRequest\x1a\x31.tensorboard.service.StreamExperimentDataResponse\"\x00\x30\x01\x62\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,tensorboard_dot_compat_dot_proto_dot_summary__pb2.DESCRIPTOR,])




_STREAMEXPERIMENTSREQUEST = _descriptor.Descriptor(
  name='StreamExperimentsRequest',
  full_name='tensorboard.service.StreamExperimentsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='read_timestamp', full_name='tensorboard.service.StreamExperimentsRequest.read_timestamp', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='user_id', full_name='tensorboard.service.StreamExperimentsRequest.user_id', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='limit', full_name='tensorboard.service.StreamExperimentsRequest.limit', index=2,
      number=3, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='experiments_mask', full_name='tensorboard.service.StreamExperimentsRequest.experiments_mask', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=146,
  serialized_end=319,
)


_STREAMEXPERIMENTSRESPONSE = _descriptor.Descriptor(
  name='StreamExperimentsResponse',
  full_name='tensorboard.service.StreamExperimentsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='experiment_ids', full_name='tensorboard.service.StreamExperimentsResponse.experiment_ids', index=0,
      number=1, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='experiments', full_name='tensorboard.service.StreamExperimentsResponse.experiments', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=321,
  serialized_end=426,
)


_EXPERIMENT = _descriptor.Descriptor(
  name='Experiment',
  full_name='tensorboard.service.Experiment',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='experiment_id', full_name='tensorboard.service.Experiment.experiment_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='create_time', full_name='tensorboard.service.Experiment.create_time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='update_time', full_name='tensorboard.service.Experiment.update_time', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_scalars', full_name='tensorboard.service.Experiment.num_scalars', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_runs', full_name='tensorboard.service.Experiment.num_runs', index=4,
      number=5, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_tags', full_name='tensorboard.service.Experiment.num_tags', index=5,
      number=6, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=429,
  serialized_end=619,
)


_EXPERIMENTMASK = _descriptor.Descriptor(
  name='ExperimentMask',
  full_name='tensorboard.service.ExperimentMask',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='create_time', full_name='tensorboard.service.ExperimentMask.create_time', index=0,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='update_time', full_name='tensorboard.service.ExperimentMask.update_time', index=1,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_scalars', full_name='tensorboard.service.ExperimentMask.num_scalars', index=2,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_runs', full_name='tensorboard.service.ExperimentMask.num_runs', index=3,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='num_tags', full_name='tensorboard.service.ExperimentMask.num_tags', index=4,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=622,
  serialized_end=758,
)


_STREAMEXPERIMENTDATAREQUEST = _descriptor.Descriptor(
  name='StreamExperimentDataRequest',
  full_name='tensorboard.service.StreamExperimentDataRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='experiment_id', full_name='tensorboard.service.StreamExperimentDataRequest.experiment_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='read_timestamp', full_name='tensorboard.service.StreamExperimentDataRequest.read_timestamp', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=760,
  serialized_end=864,
)


_STREAMEXPERIMENTDATARESPONSE_SCALARPOINTS = _descriptor.Descriptor(
  name='ScalarPoints',
  full_name='tensorboard.service.StreamExperimentDataResponse.ScalarPoints',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='steps', full_name='tensorboard.service.StreamExperimentDataResponse.ScalarPoints.steps', index=0,
      number=1, type=3, cpp_type=2, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='wall_times', full_name='tensorboard.service.StreamExperimentDataResponse.ScalarPoints.wall_times', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='values', full_name='tensorboard.service.StreamExperimentDataResponse.ScalarPoints.values', index=2,
      number=3, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1067,
  serialized_end=1160,
)

_STREAMEXPERIMENTDATARESPONSE = _descriptor.Descriptor(
  name='StreamExperimentDataResponse',
  full_name='tensorboard.service.StreamExperimentDataResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='tag_name', full_name='tensorboard.service.StreamExperimentDataResponse.tag_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='run_name', full_name='tensorboard.service.StreamExperimentDataResponse.run_name', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='tag_metadata', full_name='tensorboard.service.StreamExperimentDataResponse.tag_metadata', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='points', full_name='tensorboard.service.StreamExperimentDataResponse.points', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_STREAMEXPERIMENTDATARESPONSE_SCALARPOINTS, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=867,
  serialized_end=1160,
)

_STREAMEXPERIMENTSREQUEST.fields_by_name['read_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STREAMEXPERIMENTSREQUEST.fields_by_name['experiments_mask'].message_type = _EXPERIMENTMASK
_STREAMEXPERIMENTSRESPONSE.fields_by_name['experiments'].message_type = _EXPERIMENT
_EXPERIMENT.fields_by_name['create_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_EXPERIMENT.fields_by_name['update_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STREAMEXPERIMENTDATAREQUEST.fields_by_name['read_timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STREAMEXPERIMENTDATARESPONSE_SCALARPOINTS.fields_by_name['wall_times'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_STREAMEXPERIMENTDATARESPONSE_SCALARPOINTS.containing_type = _STREAMEXPERIMENTDATARESPONSE
_STREAMEXPERIMENTDATARESPONSE.fields_by_name['tag_metadata'].message_type = tensorboard_dot_compat_dot_proto_dot_summary__pb2._SUMMARYMETADATA
_STREAMEXPERIMENTDATARESPONSE.fields_by_name['points'].message_type = _STREAMEXPERIMENTDATARESPONSE_SCALARPOINTS
DESCRIPTOR.message_types_by_name['StreamExperimentsRequest'] = _STREAMEXPERIMENTSREQUEST
DESCRIPTOR.message_types_by_name['StreamExperimentsResponse'] = _STREAMEXPERIMENTSRESPONSE
DESCRIPTOR.message_types_by_name['Experiment'] = _EXPERIMENT
DESCRIPTOR.message_types_by_name['ExperimentMask'] = _EXPERIMENTMASK
DESCRIPTOR.message_types_by_name['StreamExperimentDataRequest'] = _STREAMEXPERIMENTDATAREQUEST
DESCRIPTOR.message_types_by_name['StreamExperimentDataResponse'] = _STREAMEXPERIMENTDATARESPONSE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

StreamExperimentsRequest = _reflection.GeneratedProtocolMessageType('StreamExperimentsRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMEXPERIMENTSREQUEST,
  '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.StreamExperimentsRequest)
  })
_sym_db.RegisterMessage(StreamExperimentsRequest)

StreamExperimentsResponse = _reflection.GeneratedProtocolMessageType('StreamExperimentsResponse', (_message.Message,), {
  'DESCRIPTOR' : _STREAMEXPERIMENTSRESPONSE,
  '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.StreamExperimentsResponse)
  })
_sym_db.RegisterMessage(StreamExperimentsResponse)

Experiment = _reflection.GeneratedProtocolMessageType('Experiment', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENT,
  '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.Experiment)
  })
_sym_db.RegisterMessage(Experiment)

ExperimentMask = _reflection.GeneratedProtocolMessageType('ExperimentMask', (_message.Message,), {
  'DESCRIPTOR' : _EXPERIMENTMASK,
  '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.ExperimentMask)
  })
_sym_db.RegisterMessage(ExperimentMask)

StreamExperimentDataRequest = _reflection.GeneratedProtocolMessageType('StreamExperimentDataRequest', (_message.Message,), {
  'DESCRIPTOR' : _STREAMEXPERIMENTDATAREQUEST,
  '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.StreamExperimentDataRequest)
  })
_sym_db.RegisterMessage(StreamExperimentDataRequest)

StreamExperimentDataResponse = _reflection.GeneratedProtocolMessageType('StreamExperimentDataResponse', (_message.Message,), {

  'ScalarPoints' : _reflection.GeneratedProtocolMessageType('ScalarPoints', (_message.Message,), {
    'DESCRIPTOR' : _STREAMEXPERIMENTDATARESPONSE_SCALARPOINTS,
    '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
    # @@protoc_insertion_point(class_scope:tensorboard.service.StreamExperimentDataResponse.ScalarPoints)
    })
  ,
  'DESCRIPTOR' : _STREAMEXPERIMENTDATARESPONSE,
  '__module__' : 'tensorboard.uploader.proto.export_service_pb2'
  # @@protoc_insertion_point(class_scope:tensorboard.service.StreamExperimentDataResponse)
  })
_sym_db.RegisterMessage(StreamExperimentDataResponse)
_sym_db.RegisterMessage(StreamExperimentDataResponse.ScalarPoints)



_TENSORBOARDEXPORTERSERVICE = _descriptor.ServiceDescriptor(
  name='TensorBoardExporterService',
  full_name='tensorboard.service.TensorBoardExporterService',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=1163,
  serialized_end=1440,
  methods=[
  _descriptor.MethodDescriptor(
    name='StreamExperiments',
    full_name='tensorboard.service.TensorBoardExporterService.StreamExperiments',
    index=0,
    containing_service=None,
    input_type=_STREAMEXPERIMENTSREQUEST,
    output_type=_STREAMEXPERIMENTSRESPONSE,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='StreamExperimentData',
    full_name='tensorboard.service.TensorBoardExporterService.StreamExperimentData',
    index=1,
    containing_service=None,
    input_type=_STREAMEXPERIMENTDATAREQUEST,
    output_type=_STREAMEXPERIMENTDATARESPONSE,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_TENSORBOARDEXPORTERSERVICE)

DESCRIPTOR.services_by_name['TensorBoardExporterService'] = _TENSORBOARDEXPORTERSERVICE

# @@protoc_insertion_point(module_scope)



# Copyright 2019 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Initial server communication to determine session parameters."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from google.protobuf import message
import requests

from tensorboard import version
from tensorboard.uploader.proto import server_info_pb2


# Request timeout for communicating with remote server.
_REQUEST_TIMEOUT_SECONDS = 10


def _server_info_request():
  request = server_info_pb2.ServerInfoRequest()
  request.version = version.VERSION
  return request


def fetch_server_info(origin):
  """Fetches server info from a remote server.

  Args:
    origin: The server with which to communicate. Should be a string
      like "https://tensorboard.dev", including protocol, host, and (if
      needed) port.

  Returns:
    A `server_info_pb2.ServerInfoResponse` message.

  Raises:
    CommunicationError: Upon failure to connect to or successfully
      communicate with the remote server.
  """
  endpoint = "%s/api/uploader" % origin
  post_body = _server_info_request().SerializeToString()
  try:
    response = requests.post(
        endpoint,
        data=post_body,
        timeout=_REQUEST_TIMEOUT_SECONDS,
        headers={"User-Agent": "tensorboard/%s" % version.VERSION},
    )
  except requests.RequestException as e:
    raise CommunicationError("Failed to connect to backend: %s" % e)
  if not response.ok:
    raise CommunicationError(
        "Non-OK status from backend (%d %s): %r"
        % (response.status_code, response.reason, response.content)
    )
  try:
    return server_info_pb2.ServerInfoResponse.FromString(response.content)
  except message.DecodeError as e:
    raise CommunicationError(
        "Corrupt response from backend (%s): %r" % (e, response.content)
    )


def create_server_info(frontend_origin, api_endpoint):
  """Manually creates server info given a frontend and backend.

  Args:
    frontend_origin: The origin of the TensorBoard.dev frontend, like
      "https://tensorboard.dev" or "http://localhost:8000".
    api_endpoint: As to `server_info_pb2.ApiServer.endpoint`.

  Returns:
    A `server_info_pb2.ServerInfoResponse` message.
  """
  result = server_info_pb2.ServerInfoResponse()
  result.compatibility.verdict = server_info_pb2.VERDICT_OK
  result.api_server.endpoint = api_endpoint
  url_format = result.url_format
  placeholder = "{{EID}}"
  while placeholder in frontend_origin:
    placeholder = "{%s}" % placeholder
  url_format.template = "%s/experiment/%s/" % (frontend_origin, placeholder)
  url_format.id_placeholder = placeholder
  return result


def experiment_url(server_info, experiment_id):
  """Formats a URL that will resolve to the provided experiment.

  Args:
    server_info: A `server_info_pb2.ServerInfoResponse` message.
    experiment_id: A string; the ID of the experiment to link to.

  Returns:
    A URL resolving to the given experiment, as a string.
  """
  url_format = server_info.url_format
  return url_format.template.replace(url_format.id_placeholder, experiment_id)


class CommunicationError(RuntimeError):
  """Raised upon failure to communicate with the server."""

  pass



# Copyright 2019 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Validates responses and their security features"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import collections

from werkzeug import wrappers
from werkzeug.datastructures import Headers
from werkzeug import http

from tensorboard.util import tb_logging

# Loosely follow vocabulary from https://www.w3.org/TR/CSP/#framework-directives.
Directive = collections.namedtuple("Directive", ["name", "value"])

logger = tb_logging.get_logger()

_HTML_MIME_TYPE = "text/html"
_CSP_DEFAULT_SRC = "default-src"
# Whitelist of allowed CSP violations.
_CSP_IGNORE = {
    # Allow TensorBoard to be iframed.
    "frame-ancestors": ["*"],
    # Polymer-based code uses unsafe-inline.
    "style-src": ["'unsafe-inline'", "data:"],
    # Used in canvas
    "img-src": ["blob:", "data:"],
    # Used by numericjs.
    # TODO(stephanwlee): remove it eventually.
    "script-src": ["'unsafe-eval'"],
}


def _maybe_raise_value_error(error_msg):
  logger.warn("In 3.0, this warning will become an error:\n%s" % error_msg)
  # TODO(3.x): raise a value error.


class SecurityValidatorMiddleware(object):
  """WSGI middleware validating security on response.

  It validates:
  - responses have Content-Type
  - responses have X-Content-Type-Options: nosniff
  - text/html responses have CSP header. It also validates whether the CSP
    headers pass basic requirement. e.g., default-src should be present, cannot
    use "*" directive, and others. For more complete list, please refer to
    _validate_csp_policies.

  Instances of this class are WSGI applications (see PEP 3333).
  """

  def __init__(self, application):
    """Initializes an `SecurityValidatorMiddleware`.

    Args:
      application: The WSGI application to wrap (see PEP 3333).
    """
    self._application = application

  def __call__(self, environ, start_response):

    def start_response_proxy(status, headers, exc_info=None):
      self._validate_headers(headers)
      return start_response(status, headers, exc_info)

    return self._application(environ, start_response_proxy)

  def _validate_headers(self, headers_list):
    headers = Headers(headers_list)
    self._validate_content_type(headers)
    self._validate_x_content_type_options(headers)
    self._validate_csp_headers(headers)

  def _validate_content_type(self, headers):
    if headers.get("Content-Type"):
      return

    _maybe_raise_value_error("Content-Type is required on a Response")

  def _validate_x_content_type_options(self, headers):
    option = headers.get("X-Content-Type-Options")
    if option == "nosniff":
      return

    _maybe_raise_value_error(
        'X-Content-Type-Options is required to be "nosniff"'
    )

  def _validate_csp_headers(self, headers):
    mime_type, _ = http.parse_options_header(headers.get("Content-Type"))
    if mime_type != _HTML_MIME_TYPE:
      return

    csp_texts = headers.get_all("Content-Security-Policy")
    policies = []

    for csp_text in csp_texts:
      policies += self._parse_serialized_csp(csp_text)

    self._validate_csp_policies(policies)

  def _validate_csp_policies(self, policies):
    has_default_src = False
    violations = []

    for directive in policies:
      name = directive.name
      for value in directive.value:
        has_default_src = has_default_src or name == _CSP_DEFAULT_SRC

        if value in _CSP_IGNORE.get(name, []):
          # There are cases where certain directives are legitimate.
          continue

        # TensorBoard follows principle of least privilege. However, to make it
        # easier to conform to the security policy for plugin authors,
        # TensorBoard trusts request and resources originating its server. Also,
        # it can selectively trust domains as long as they use https protocol.
        # Lastly, it can allow 'none' directive.
        # TODO(stephanwlee): allow configuration for whitelist of domains for
        # stricter enforcement.
        # TODO(stephanwlee): deprecate the sha-based whitelisting.
        if (
            value == "'self'" or value == "'none'"
            or value.startswith("https:")
            or value.startswith("'sha256-")
        ):
          continue

        msg = "Illegal Content-Security-Policy for {name}: {value}".format(
            name=name, value=value
        )
        violations.append(msg)

    if not has_default_src:
      violations.append("Requires default-src for Content-Security-Policy")

    if violations:
      _maybe_raise_value_error("\n".join(violations))

  def _parse_serialized_csp(self, csp_text):
    # See https://www.w3.org/TR/CSP/#parse-serialized-policy.
    # Below Steps are based on the algorithm stated in above spec.
    # Deviations:
    # - it does not warn and ignore duplicative directive (Step 2.5)

    # Step 2
    csp_srcs = csp_text.split(";")

    policy = []
    for token in csp_srcs:
      # Step 2.1
      token = token.strip()

      if not token:
        # Step 2.2
        continue

      # Step 2.3
      token_frag = token.split(None, 1)
      name = token_frag[0]

      values = token_frag[1] if len(token_frag) == 2 else ""

      # Step 2.4
      name = name.lower()

      # Step 2.6
      value = values.split()
      # Step 2.7
      directive = Directive(name=name, value=value)
      # Step 2.8
      policy.append(directive)

    return policy



# Copyright 2019 The TensorFlow Authors. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ==============================================================================
"""Provides a lazy wrapper for deferring Tensor creation."""

import threading

from tensorboard.compat import tf2 as tf


# Sentinel used for LazyTensorCreator._tensor to indicate that a value is
# currently being computed, in order to fail hard on reentrancy.
_CALL_IN_PROGRESS_SENTINEL = object()


class LazyTensorCreator(object):
  """Lazy auto-converting wrapper for a callable that returns a `tf.Tensor`.

  This class wraps an arbitrary callable that returns a `Tensor` so that it
  will be automatically converted to a `Tensor` by any logic that calls
  `tf.convert_to_tensor()`. This also memoizes the callable so that it is
  called at most once.

  The intended use of this class is to defer the construction of a `Tensor`
  (e.g. to avoid unnecessary wasted computation, or ensure any new ops are
  created in a context only available later on in execution), while remaining
  compatible with APIs that expect to be given an already materialized value
  that can be converted to a `Tensor`.

  This class is thread-safe.
  """

  def __init__(self, tensor_callable):
    """Initializes a LazyTensorCreator object.

    Args:
      tensor_callable: A callable that returns a `tf.Tensor`.
    """
    if not callable(tensor_callable):
      raise ValueError("Not a callable: %r" % tensor_callable)
    self._tensor_callable = tensor_callable
    self._tensor = None
    self._tensor_lock = threading.RLock()
    _register_conversion_function_once()

  def __call__(self):
    if self._tensor is None or self._tensor is _CALL_IN_PROGRESS_SENTINEL:
      with self._tensor_lock:
        if self._tensor is _CALL_IN_PROGRESS_SENTINEL:
          raise RuntimeError("Cannot use LazyTensorCreator with reentrant callable")
        elif self._tensor is None:
          self._tensor = _CALL_IN_PROGRESS_SENTINEL
          self._tensor = self._tensor_callable()
    return self._tensor


def _lazy_tensor_creator_converter(value, dtype=None, name=None, as_ref=False):
  del name  # ignored
  if not isinstance(value, LazyTensorCreator):
    raise RuntimeError("Expected LazyTensorCreator, got %r" % value)
  if as_ref:
    raise RuntimeError("Cannot use LazyTensorCreator to create ref tensor")
  tensor = value()
  if dtype not in (None, tensor.dtype):
    raise RuntimeError(
        "Cannot convert LazyTensorCreator returning dtype %s to dtype %s" % (
            tensor.dtype, dtype))
  return tensor


# Use module-level bit and lock to ensure that registration of the
# LazyTensorCreator conversion function happens only once.
_conversion_registered = False
_conversion_registered_lock = threading.Lock()


def _register_conversion_function_once():
  """Performs one-time registration of `_lazy_tensor_creator_converter`.

  This helper can be invoked multiple times but only registers the conversion
  function on the first invocation, making it suitable for calling when
  constructing a LazyTensorCreator.

  Deferring the registration is necessary because doing it at at module import
  time would trigger the lazy TensorFlow import to resolve, and that in turn
  would break the delicate `tf.summary` import cycle avoidance scheme.
  """
  global _conversion_registered
  if not _conversion_registered:
    with _conversion_registered_lock:
      if not _conversion_registered:
        _conversion_registered = True
        tf.register_tensor_conversion_function(
            base_type=LazyTensorCreator,
            conversion_func=_lazy_tensor_creator_converter,
            priority=0)
